/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.portal.search;

import com.liferay.portal.kernel.messaging.DestinationNames;
import com.liferay.portal.kernel.messaging.MessageBusUtil;
import com.liferay.portal.kernel.search.Document;
import com.liferay.portal.kernel.search.IndexWriter;
import com.liferay.portal.kernel.search.messaging.SearchRequest;

/**
 * <a href="IndexWriterImpl.java.html"><b><i>View Source</i></b></a>
 *
 * @author Bruno Farache
 *
 */
public class IndexWriterImpl implements IndexWriter {

	public void addDocument(long companyId, Document doc) {
		SearchRequest searchRequest = new SearchRequest();

		searchRequest.setCommand(SearchRequest.COMMAND_ADD);
		searchRequest.setCompanyId(companyId);
		searchRequest.setDocument(doc);

		MessageBusUtil.sendMessage(
			DestinationNames.SEARCH_WRITER, searchRequest);
	}

	public void deleteDocument(long companyId, String uid) {
		SearchRequest searchRequest = new SearchRequest();

		searchRequest.setCommand(SearchRequest.COMMAND_DELETE);
		searchRequest.setCompanyId(companyId);
		searchRequest.setId(uid);

		MessageBusUtil.sendMessage(
			DestinationNames.SEARCH_WRITER, searchRequest);
	}

	public void deletePortletDocuments(long companyId, String portletId) {
		SearchRequest searchRequest = new SearchRequest();

		searchRequest.setCommand(SearchRequest.COMMAND_DELETE_PORTLET_DOCS);
		searchRequest.setCompanyId(companyId);
		searchRequest.setId(portletId);

		MessageBusUtil.sendMessage(
			DestinationNames.SEARCH_WRITER, searchRequest);
	}

	public void updateDocument(long companyId, String uid, Document doc) {
		SearchRequest searchRequest = new SearchRequest();

		searchRequest.setCommand(SearchRequest.COMMAND_UPDATE);
		searchRequest.setCompanyId(companyId);
		searchRequest.setId(uid);
		searchRequest.setDocument(doc);

		MessageBusUtil.sendMessage(
			DestinationNames.SEARCH_WRITER, searchRequest);
	}

}