/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.portal.spring.annotation;

import com.liferay.portal.kernel.annotation.BeanReference;
import com.liferay.portal.kernel.bean.BeanLocatorException;
import com.liferay.portal.kernel.bean.PortalBeanLocatorUtil;

import java.beans.PropertyDescriptor;

import java.lang.reflect.AnnotatedElement;
import java.lang.reflect.Member;

import org.springframework.beans.BeansException;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.annotation.InjectionMetadata;

/**
 * <a href="BeanReferenceElement.java.html"><b><i>View Source</i></b></a>
 *
 * @author Michael Young
 * @author Raymond Augé
 *
 */
public class BeanReferenceElement extends InjectionMetadata.InjectedElement {

	public BeanReferenceElement(
		BeanFactory beanFactory, Member member,
		PropertyDescriptor propertyDescriptor) {

		super(member, propertyDescriptor);

		_beanFactory = beanFactory;

		AnnotatedElement annotatedElement = (AnnotatedElement)member;

		BeanReference beanReference = annotatedElement.getAnnotation(
			BeanReference.class);

		_name = beanReference.name();
	}

	protected Object getResourceToInject(
		Object target, String requestingBeanName) {

		try {
			return _beanFactory.getBean(_name, getResourceType());
		}
		catch (BeansException be) {
			try {
				return PortalBeanLocatorUtil.locate(_name);
			}
			catch (BeanLocatorException ble) {
				throw be;
			}
		}
	}

	private BeanFactory _beanFactory;
	private String _name;

}