/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.portal.struts;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.servlet.BrowserSnifferUtil;

import java.io.IOException;

import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.portlet.PortletContext;
import javax.portlet.PortletRequest;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.PageContext;

import org.apache.struts.Globals;

/**
 * <a href="StrutsUtil.java.html"><b><i>View Source</i></b></a>
 *
 * @author Brian Wing Shun Chan
 *
 */
public class StrutsUtil {

	public static final String STRUTS_PACKAGE = "org.apache.struts.";

	public static final String TEXT_HTML_DIR = "/html";

	public static final String TEXT_WAP_DIR = "/wap";

	public static void forward(
			String uri, ServletContext servletContext,
			HttpServletRequest request, HttpServletResponse response)
		throws ServletException {

		if (_log.isDebugEnabled()) {
			_log.debug("Forward URI " + uri);
		}

		if (uri.equals(ActionConstants.COMMON_NULL)) {
			return;
		}

		if (!response.isCommitted()) {
			String path = TEXT_HTML_DIR + uri;

			if (BrowserSnifferUtil.isWap(request)) {
				path = TEXT_WAP_DIR + uri;
			}

			if (_log.isDebugEnabled()) {
				_log.debug("Forward path " + path);
			}

			RequestDispatcher requestDispatcher =
				servletContext.getRequestDispatcher(path);

			try {
				requestDispatcher.forward(request, response);
			}
			catch (IOException ioe1) {
				_log.warn(ioe1, ioe1);
			}
			catch (ServletException se1) {
				request.setAttribute(PageContext.EXCEPTION, se1.getRootCause());

				String errorPath = TEXT_HTML_DIR + "/common/error.jsp";

				if (BrowserSnifferUtil.isWap(request)) {
					path = TEXT_WAP_DIR + "/common/error.jsp";
				}

				requestDispatcher = servletContext.getRequestDispatcher(
					errorPath);

				try {
					requestDispatcher.forward(request, response);
				}
				catch (IOException ioe2) {
					_log.warn(ioe2, ioe2);
				}
				catch (ServletException se2) {
					throw se2;
				}
			}
		}
		else {
			_log.warn(uri + " is already committed");
		}
	}

	public static void include(
			String uri, ServletContext servletContext,
			HttpServletRequest request, HttpServletResponse response)
		throws ServletException {

		if (_log.isDebugEnabled()) {
			_log.debug("Include URI " + uri);
		}

		String path = TEXT_HTML_DIR + uri;

		if (BrowserSnifferUtil.isWap(request)) {
			path = TEXT_WAP_DIR + uri;
		}

		if (_log.isDebugEnabled()) {
			_log.debug("Include path " + path);
		}

		RequestDispatcher requestDispatcher =
			servletContext.getRequestDispatcher(path);

		try {
			requestDispatcher.include(request, response);
		}
		catch (IOException ioe) {
			_log.warn(ioe, ioe);
		}
	}

	public static Map<String, Object> removeStrutsAttributes(
		PortletContext portletContext, PortletRequest portletRequest) {

		Map<String, Object> strutsAttributes = new HashMap<String, Object>();

		Enumeration<String> enu = portletRequest.getAttributeNames();

		while (enu.hasMoreElements()) {
			String attributeName = enu.nextElement();

			if (attributeName.startsWith(STRUTS_PACKAGE)) {
				strutsAttributes.put(
					attributeName, portletRequest.getAttribute(attributeName));
			}
		}

		Iterator<String> itr = strutsAttributes.keySet().iterator();

		while (itr.hasNext()) {
			String attributeName = itr.next();

			portletRequest.setAttribute(attributeName, null);
		}

		Object moduleConfig = portletContext.getAttribute(Globals.MODULE_KEY);

		portletRequest.setAttribute(Globals.MODULE_KEY, moduleConfig);

		return strutsAttributes;
	}

	public static void setStrutsAttributes(
		PortletRequest portletRequest, Map<String, Object> strutsAttributes) {

		for (Map.Entry<String, Object> entry : strutsAttributes.entrySet()) {
			String key = entry.getKey();
			Object value = entry.getValue();

			portletRequest.setAttribute(key, value);
		}
	}

	private static Log _log = LogFactoryUtil.getLog(StrutsUtil.class);

}